#include "Background.h"
#include "ResourceManager.h"
#include "EngineError.h"
#include "BloodEffect.h"
#include <SDL/SDL.h>
#include <GL/gl.h>
#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH

const char BACKGROUND_PATH[] = "data/ground.bmp";

// the intensity of the blood on the ground will be limited
// by the following value
const float GROUNDBLOOD_MAX_INTENSITY = 0.85;

Background::Background()
{
	backgroundTexture = ResourceManager::get().getTexture(BACKGROUND_PATH);

	for (int i = 0; i < Engine::WORLDWIDTH; ++i)
	{
		for (int j = 0; j < Engine::WORLDHEIGHT; ++j)
			bloodpixels[i][j] = 0;
	}
}


void Background::draw() const
{
	drawBackgroundTexture();
	drawBlood();
}


void Background::addBlood(int x, int y, float intensity)
{
	const int filterfactor = BloodEffect::FILTERFACTOR;

	// we filter over a 3x3 area
	addBloodImpl(x - 2, y - 2, intensity / filterfactor / 2);
	addBloodImpl(x - 1, y - 2, intensity / filterfactor / 2);
	addBloodImpl(x, y - 2, intensity / filterfactor / 2);
	addBloodImpl(x + 1, y - 2, intensity / filterfactor / 2);
	addBloodImpl(x + 2, y - 2, intensity / filterfactor / 2 );

	addBloodImpl(x - 2, y - 1, intensity / filterfactor / 2);
	addBloodImpl(x - 1, y - 1, intensity / filterfactor);
	addBloodImpl(x, y - 1, intensity / filterfactor);
	addBloodImpl(x + 1, y - 1, intensity / filterfactor);
	addBloodImpl(x + 2, y - 1, intensity / filterfactor / 2 );

	addBloodImpl(x - 2, y, intensity / filterfactor / 2 );
	addBloodImpl(x - 1, y, intensity / filterfactor);
	addBloodImpl(x, y, intensity);
	addBloodImpl(x + 1, y, intensity / filterfactor);
	addBloodImpl(x + 2, y, intensity / filterfactor / 2 );

	addBloodImpl(x - 2, y + 2, intensity / filterfactor / 2);
	addBloodImpl(x - 1, y + 2, intensity / filterfactor / 2);
	addBloodImpl(x, y + 2, intensity / filterfactor / 2);
	addBloodImpl(x + 1, y + 2, intensity / filterfactor / 2);
	addBloodImpl(x + 2, y + 2, intensity / filterfactor / 2);
}

void Background::addBloodImpl(int x, int y, float intensity)
{
	if (x < 0 || x >= Engine::WORLDWIDTH) return;
	if (y < 0 || y >= Engine::WORLDHEIGHT) return;

	bloodpixels[x][y] += intensity;

	if (bloodpixels[x][y] >= GROUNDBLOOD_MAX_INTENSITY)
		bloodpixels[x][y] = GROUNDBLOOD_MAX_INTENSITY;
}


void Background::drawBackgroundTexture() const
{
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glBindTexture(GL_TEXTURE_2D, backgroundTexture);

	glBegin(GL_QUADS);
	glTexCoord2f(1, 1);
	glVertex2f(0, 0);
	glTexCoord2f(1, 0);
	glVertex2f(Engine::WORLDWIDTH, 0);
	glTexCoord2f(0, 0);
	glVertex2f(Engine::WORLDWIDTH, Engine::WORLDHEIGHT);
	glTexCoord2f(0, 1);
	glVertex2f(0, Engine::WORLDHEIGHT);
	glEnd();
	glDisable(GL_TEXTURE_2D);
}


void Background::drawBlood() const
{
	glBegin(GL_POINTS);

	for (int i = 0; i < Engine::WORLDWIDTH; ++i)
	{
		for (int j = 0; j < Engine::WORLDHEIGHT; ++j)
		{
			if (bloodpixels[i][j] < 0.05)
				continue;
			glColor4f(1, 0, 0, bloodpixels[i][j]);
			glVertex2i(i, j);
		}
	}

	glEnd();
}

