#ifndef BACKGROUND_H
#define BACKGROUND_H

/**
 * Background.h: contains the Background-class.
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "Engine.h"
#include "typedefs.h"


/**
 * The Background of the game.
 *
 * Basically contains a background-texture. Also stores
 * all the blood that gets spilled on the ground.
 */
class Background
{
	public:
		Background();
		void draw() const;

		/**
		 * Adds blood to the position (x/y).
		 * Intensity is a value between 0-1.
		 */
		void addBlood(int x, int y, float intensity);

	private:
		struct BloodPixel
		{
			int x;
			int y;
			float intensity;
			BloodPixel(int x_, int y_, float i)
				: x(x_), y(y_), intensity(i)
				{}
		};

		float bloodpixels[Engine::WORLDWIDTH][Engine::WORLDHEIGHT];

		Texture backgroundTexture;

		void drawBackgroundTexture() const;
		void drawBlood() const;

		void addBloodImpl(int x, int y, float intensity);
};

#endif // BACKGROUND_H
