#ifndef BLOODEFFECT_H
#define BLOODEFFECT_H

/**
 * BloodEffect.h: the class that is responsible for the blood effects
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "Effect.h"
#include "Vector3D.h"
#include <vector>

class Vector2D;


/**
 * The BloodEffect is basically a particle engine generating the effects
 * when a zombie gets hit by a shot.
 */
class BloodEffect : public Effect
{
	public:
		struct Particle
		{
			Vector3D pos;
			Vector3D vel;
		};

		typedef std::vector<Particle> ParticleContainer;
		typedef ParticleContainer::iterator ParticleIterator;

		BloodEffect(const Vector2D& pos, const Vector2D& dir);
		~BloodEffect();

		virtual void update();
		virtual void draw(float dt);
		virtual bool isFinished();


		/**
		 * How much the bloodpixels will be "filtered",
		 * i.e.with what intensity the pixels next to the bloodpixel
		 * will be drawn. (intensity of pixels next to the one
		 * = origininal intensity / FILTERFACTOR
		 */
		static const int FILTERFACTOR = 4;

	private:
		ParticleContainer particles;
};

#endif // BLOODEFFECTS_H
