/**
 * Clock.cpp:
 * 	a SDL-specific implementation of Clock.h
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "Clock.h"
#include <SDL/SDL_timer.h>


const int Clock::LOGICFRAMES_PER_SECOND = 20;
const int TICKTIME = 1000/Clock::LOGICFRAMES_PER_SECOND;


Clock& Clock::get()
{
	static Clock c;
	return c;
}


Clock::Clock()
	:  nLogicTicks(0), frameIdx(0)
{
	nextLogicTime = getTicks();
}


Time Clock::getTicks() const
{
	return SDL_GetTicks();
}


bool Clock::needLogicTick() const
{
	Time now = getTicks();
//	printf("?now: %u, next: %u	%i\n", now, nextLogicTime, (nextLogicTime < now));
	return nextLogicTime < now;
}


void Clock::newLogicTick()
{
	nextLogicTime += TICKTIME;
	++nLogicTicks;
//	printf("!now: %u, next: %u\n", GetTickCount(), nextLogicTime);
//	lastLogicTime = GetTickCount();
}

void Clock::newDrawTick()
{
	Time now = getTicks();
	frames[frameIdx++] = now - lastDrawTick;
	if (frameIdx >= FRAME_COUNT)
		frameIdx -= FRAME_COUNT;
	lastDrawTick = now;
}


float Clock::getFps() const
{
	unsigned int sum = 0;
	for (size_t i = 0; i < FRAME_COUNT; ++i)
		sum += frames[i];

	return 1000.f / (static_cast<float>(sum) / FRAME_COUNT);
}


float Clock::getProgress() const
{
	Time now = getTicks();
	int diff = static_cast<signed>(nextLogicTime - now);
	float ratio = static_cast<float>(diff) / TICKTIME;
	return 1.f - ratio;
}
