#ifndef CLOCK_H
#define CLOCK_H

/**
 * Clock.h: a basic clock class
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "typedefs.h"
#include <cstddef>

/**
 * The basic clock of the game.
 * The Clock is responsible for calculating FPS and
 * providing the timing for the game (making sure logic runs at
 * constant logicticks/seconds).
 */
class Clock
{
	public:
		/** Returns true if it's time for a logic tick */
		bool needLogicTick() const;

		/** Call this at the beginning of every new logictick */
		void newLogicTick();

		/** Call this at the beginning of every new drawing tick */
		void newDrawTick();

		/**
		 * Returns (in percent) how much of the current logic tick already passed.
		 * I.E. directly after a new tick this returns 0, directly befor it returns 1,
		 * and 0.5 will be returned if LOGICFRAMES_PER_SECOND/2 time has passed
		 * since the last tick.
		 */
		float getProgress() const;
		float getFps() const;

		Time getLogicTicks() const {return nLogicTicks;}
		Time getAbsoluteTicks() const {return getTicks();}

		static Clock& get();

		static const int LOGICFRAMES_PER_SECOND;

	private:
		Time getTicks() const;

		Clock();
		Time nextLogicTime;
		Time lastDrawTick;
		Time nLogicTicks;

		// for calculating the fps
		// FRAME_COUNT defines how many frames will be considered when calculating the fps
		static const size_t FRAME_COUNT = 15;
		unsigned int frames[FRAME_COUNT];
		size_t frameIdx;
};



#endif // CLOCK_H
