#include "Cursor.h"

#include <SDL/SDL.h>
#include <cstring>
#include "EngineError.h"
#include "../rawdata/shooting_cursor.xpm"

namespace
{
/*
 * Converts an XPM-Image to a Cursor.
 * Taken from
 * The official SDL-documentation under SDL_CreateCursor
 */
SDL_Cursor* _loadCursor(const char *image[])
{
	int i, row, col;
	int width, height, colors;
	sscanf(image[0], "%d %d %d", &width, &height, &colors);

	// TODO: haven't figured out what the 32 stands for here ?-)
	int size = width*height / 8;
	Uint8* data = new Uint8[size];
	Uint8* mask = new Uint8[size];

	memset(data, 0, size);
	memset(mask, 0, size);

	i = -1;
	for (row = 0; row < height; ++row)
	{
		for (col = 0; col < width; ++col)
		{
			if (col % 8)
			{
				data[i] <<= 1;
				mask[i] <<= 1;
			}
			else
				++i;

			switch (image[colors+ 1 + row][col])
			{
				case 'X':
					data[i] |= 0x01;
					mask[i] |= 0x01;
					break;
				case '.':
					mask[i] |= 0x01;
					break;
				case ' ':
					data[i] |= 0x0;
					mask[i] |= 0x0;
				break;
			}
		}
	}

	int hot_x, hot_y;
	sscanf(image[colors + 1 + row], "%d,%d", &hot_x, &hot_y);
	return SDL_CreateCursor(data, mask, width, height, hot_x, hot_y);
}
}

Cursor::Cursor()
{
	shootingCursor = _loadCursor(shooting_cursor);
	if (!shootingCursor)
		throw EngineError("Unable to load cursor");
}

Cursor::~Cursor()
{
	SDL_FreeCursor(shootingCursor);
}


void Cursor::setShootingCursor()
{
	SDL_SetCursor(shootingCursor);
}


