#ifndef EFFECT_H
#define EFFECT_H


/**
 * Effect.h: abstract baseclass for Effects.
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "Animator.h"


/**
 * An Effect is a special kind of Animator (hence, it draws itself)
 * Effects are registered to the Engine and to the Renderer (this
 * functionality is provided in the Ctor/Dtor of this Baseclass).
 *
 * Every effect needs to return true on isFinished() once the effect
 * has "finished" and can be removed from the game.
 *
 */
class Effect : public Animator
{
	public:
		Effect();
		virtual ~Effect();

		/** Returns true once the effect is over */
		virtual bool isFinished() = 0;
		virtual void draw(float dt) = 0;
		virtual void update() = 0;
};


#endif
