#ifndef ENGINE_H
#define ENGINE_H

/**
 * Engine.h: a simple engine.
 *
 * (c) 2007 Thomas Unterthiner
 *
 */


#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <list>
class SpriteManager;
class ResourceManager;
class Game;
class Effect;

typedef boost::shared_ptr<Effect> EffectPtr;

/**
 * A simple Engine for a game.
 */
class Engine
{
	public:
		~Engine();

		/*
		 * Effects have to be managed by the Engine and not by the
		 * Game, because some effects go on even if the game is over
		 * (i.e. the game-over effect).
		 */
		typedef std::list<EffectPtr> EffectList;
		void registerEffect(Effect* effect);

		void run();
		const SpriteManager& getSpriteManager() const {return *spriteManager;}
		const ResourceManager& getResourceManager() const {return *resourceManager;}

		static Engine& get();

		static const int WORLDWIDTH = 800;
		static const int WORLDHEIGHT = 600;

	private:
		Engine();
		void setup();
		void tearDown();
		void update();
		void handleInput();
		void handleKeypress();
		void loadPawns();
		void handleEffects();

		boost::scoped_ptr<SpriteManager> spriteManager;
		boost::scoped_ptr<ResourceManager> resourceManager;
		boost::scoped_ptr<Game> game;
		EffectList effects;
		bool isRunning;
		int windowWidth;
		int windowHeight;
};

#endif
