#include "Game.h"
#include "Engine.h"
#include "Zombie.h"
#include "GameObject.h"
#include "GameOverEffect.h"
#include "PlayerPawn.h"
#include "Clock.h"
#include "Sample.h"
#include <cstdlib>
//#include <iostream>

const float INITIAL_ZOMBIE_SPAWNRATE = 1.5;  // in Zombies per second
const float INITIAL_ZOMBIE_SPEED = 1.5;
const unsigned int INITIAL_ZOMBIE_HITPOINTS = 5;

Game* Game::instance = 0;

Game::Game()
	: isRunning_(false),
	  lastZombieSpawn(0),
	  zombieSpawnRate(INITIAL_ZOMBIE_SPAWNRATE),
	  zombieSpeed(INITIAL_ZOMBIE_SPEED),
	  zombieHitpoints(INITIAL_ZOMBIE_HITPOINTS),
	  deadZombies(0),
	  levelupSound(LEVEL_UP),
	  swearSound(PLAYER_SWEAR)
{
//	std::cout << "game " << this << " born" << std::endl;
}

Game::~Game()
{
//	std::cout << "game " << this << " dying" << std::endl;
	playerPawn.reset();
	objects.clear();
}

void Game::start()
{
//	std::cout << "creating new playerpawn" << std::endl;
	Vector2D pos(Engine::WORLDWIDTH/2, Engine::WORLDHEIGHT/2);
	playerPawn.reset(new PlayerPawn(pos));
	isRunning_ = true;
}


void Game::registerObject(GameObject* object)
{
	objects.push_back(GameObjectPtr(object));
}

void Game::registerZombie(Zombie* zombie)
{
	zombies.push_back(zombie);
	registerObject(zombie);
}

/*
void Game::unregisterObject(GameObject* object)
{
	for (ObjectList::iterator i = objects.begin(); i != objects.end(); ++i)
	{
		if (i->get() == object)
			i = objects.erase(i);
	}
}
*/

void Game::unregisterZombie(Zombie* zombie)
{
	zombies.remove(zombie);
}

void Game::onGameOver()
{
	Effect* go = new GameOverEffect();
	Engine::get().registerEffect(go);
	isRunning_ = false;
}

void Game::onZombieDeath(Zombie* zombie)
{
	scoredisplay.addPoints(1);

	if (rand() % 75 == 1) // swear every once in a while
		swearSound.play();

	++deadZombies;
	if (deadZombies % 100 == 0)
	{
		levelUp();
	}
}


void Game::levelUp()
{
	zombieSpawnRate *= 1.2;
	zombieSpeed *= 1.2;
	zombieHitpoints += 2;
	levelupSound.play();
}


void Game::update()
{
	for (ObjectList::iterator i = objects.begin(); i != objects.end(); ++i)
	{
		(*i)->update();
		if (!(*i)->isAlive())
			i = objects.erase(i);
	}
	playerPawn->update();
	scoredisplay.update();
	spawnZombiesIfNecessary();
}


void Game::spawnZombiesIfNecessary()
{
	Time ticksSinceZombie = Clock::get().getLogicTicks() - lastZombieSpawn;
	if (ticksSinceZombie >= Clock::LOGICFRAMES_PER_SECOND / zombieSpawnRate)
	{
		float startX = (rand() & 1) ? 0 : Engine::WORLDWIDTH;
		float startY =  (rand() & 1) ? 0 : Engine::WORLDHEIGHT;
		if (rand() & 1)
			startX = rand() % Engine::WORLDWIDTH;
		else
			startY = rand() % Engine::WORLDHEIGHT;

		Vector2D startPos(startX, startY);

		Zombie* zombie(new Zombie(startPos, zombieSpeed, zombieHitpoints));
		registerZombie(zombie);
		lastZombieSpawn = Clock::get().getLogicTicks();
	}
}
