#ifndef GAME_H
#define GAME_H

/**
 * Game.h: contains all of the gamelogic
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "Scoredisplay.h"
#include "Clock.h"
#include "Sound.h"
#include <list>
#include <boost/shared_ptr.hpp>
#include <boost/scoped_ptr.hpp>

class GameObject;
class Zombie;
class PlayerPawn;
class Engine;

typedef boost::shared_ptr<GameObject> GameObjectPtr;
typedef boost::shared_ptr<Zombie> ZombiePtr;


/**
 * The Game-class manages all the things related to the
 * actual gamelogic.
 * Its main method is "update" which gets called once every
 * logictick and actually does all the work in the game.
 */
class Game
{
	public:
		~Game();
		typedef std::list<GameObjectPtr> ObjectList;
		typedef std::list<Zombie*> ZombieList;

		void registerObject(GameObject* object);
		void registerZombie(Zombie* zombie);
		void unregisterZombie(Zombie* zombie);
		PlayerPawn& getPlayerPawn() const {return *playerPawn;}
		const ObjectList& getObjectList() const {return objects;}
		const ZombieList& getZombieList() const {return zombies;}

		void start();
		void update();
		bool isRunning() const {return isRunning_;}

		void onZombieDeath(Zombie* zombie); // called when a zombie dies
		void onGameOver(); // called when the Player dies

		static Game& get() {return *instance;}


	private:
		void spawnZombiesIfNecessary();
		void levelUp();

		ObjectList objects;
		ZombieList zombies;
		boost::scoped_ptr<PlayerPawn> playerPawn;
		Scoredisplay scoredisplay;
		bool isRunning_;

		Time lastZombieSpawn;
		float zombieSpawnRate; // in Zombies per second
		float zombieSpeed;
		unsigned int zombieHitpoints;
		unsigned int deadZombies;
		Sound levelupSound;
		Sound swearSound;

		// only the engine can create new games
		friend class Engine;
		Game();
		static Game* instance;

};



#endif // GAME_H
