#ifndef GAMEOBJECT_H
#define GAMEOBJECT_H

/**
 * GameObject.h: contains the baseclass for all the objects in the game
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "Vector2D.h"
#include "Animator.h"
#include <memory>

/**
 * The GameObject is the baseclass for all the objects that interact in
 * the game, as the Player, the Zombies, the Bloodeffects, ...
 * All the GameObjects have an Animator (that are responsible for painting
 * them). On every logic-tick the update()-function of the GO is called so
 * that he can "think" what he will do next.
 */
class GameObject
{
	public:
		typedef std::auto_ptr<Animator> AnimatorPtr;

		GameObject(AnimatorPtr animator, const Vector2D& pos, const Vector2D& size);
		virtual ~GameObject();
		const Vector2D& getPosition() const {return pos;}
		const Vector2D& getSize() const {return size;}

		virtual Vector2D calculateNextPosition();

		/** Update logic. */
		virtual void update();

		/** Returns false if the object is dead and can be removed from the game */
		virtual bool isAlive() {return true;};
	protected:
		Vector2D pos;  // position of the lower-left corner of the object!
		Vector2D size;
		AnimatorPtr animator;
};



#endif
