#include "Renderer.h"
#include "EngineError.h"
#include "Engine.h"
#include "Animator.h"
#include "ResourceManager.h"
#include "Background.h"
#include <SDL/SDL.h>
#include <GL/gl.h>
#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH

using namespace boost;

const float CLEAR_COLOR[] = {0.5f, 1.0f, 0.5f,};

Renderer* Renderer::instance = 0;

Renderer::Renderer()
{

}

Renderer::~Renderer()
{
	animators.clear();
	SDL_QuitSubSystem(SDL_INIT_VIDEO);
}

void Renderer::setup(int width, int height)
{
	if (!(SDL_WasInit(SDL_INIT_VIDEO) & SDL_INIT_VIDEO))
	{
		if (SDL_InitSubSystem(SDL_INIT_VIDEO) < 0)
			throw EngineError("unable to initialize the video subsystem");
	}
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1);
	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8);

	if (SDL_SetVideoMode(width, height, 0, SDL_OPENGL) == NULL)
		throw EngineError("unable to initialize OpenGL");

	glClearColor(CLEAR_COLOR[0], CLEAR_COLOR[1], CLEAR_COLOR[2], 0);
	glClearDepth(1.0);
//	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_FLAT);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	resize(width, height);

	ResourceManager::get().loadResources();

	// we can create the background only after initiating SDL
	// because we need to load some textures in the Background's Ctor
	background.reset(new Background());
}


void Renderer::resize(int width, int height) const
{
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	// bloodparticles might fly as high as z = 38
	glOrtho(0, width, 0, height, -50, 0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}


void Renderer::registerAnimator(Animator* a)
{
	// we use push front, so the sooner an animator
	// is registered, the later it will be drawn.
	// the playerpawn is the first one to register itself
	// so it will be drawn last (ie: on top of all the
	// others).
	animators.push_front(a);
}


void Renderer::unregisterAnimator(Animator* a)
{
	animators.remove(a);
}


void Renderer::registerAnimatorOnTop(Animator* a)
{
	animators.push_back(a);
}


void Renderer::draw(float dt) const
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	background->draw();
	foreach (Animator* a, animators)
	{
		a->draw(dt);
	}

	SDL_GL_SwapBuffers();
}
