#ifndef RENDERER_H
#define RENDERER_H

#include <list>
#include <memory>
#include "Background.h"

class Engine;
class Animator;

class Renderer
{
	public:
		~Renderer();
		void resize(int width, int height) const;
		void draw(float dt) const;
		void registerAnimator(Animator* a);

		/**
		 * Registers an animator that should be drawn last.
		 * An Animator registered with this function has a greater
		 * chance of being drawn last (i.e. on top of all the others).
		 * (Only the last Animator to be registered this way gets
		 * drawn last, of course).
		 */
		void registerAnimatorOnTop(Animator* a);
		void unregisterAnimator(Animator* a);

		Background& getBackground() {return *background;}
		static Renderer& get() {return *instance;}

	private:
		std::list<Animator*> animators;
		std::auto_ptr<Background> background;

		// the Engine creates the only renderer instance and deletes it
		// check the Engine::tearDown for details
		friend class Engine;
		static Renderer* instance;

		Renderer();
		void setup(int width, int height);
};

#endif
