#ifndef RESOURCEHANDLER_H
#define RESOURCEHANDLER_H

/**
 * ResourceManager.h: includes the ResourceManager
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "typedefs.h"
#include "Sample.h"
#include <string>
#include <vector>
#include <map>

class SDL_Surface;
class SDL_Rect;
class SDL_RWops;
typedef std::vector<Texture> TextureArray;


/**
 * The ResourceManager is responsible for handling all
 * the Resources, i.e. loading Sprites, Textures, ...
 * Also it kind of abstracts a virtual file system, so all
 * the resources can be loaded by aksing for the appropriate
 * filepath, but it's up to the RM if the file is really loaded
 * from that path of from somewhere else.
 */
class ResourceManager
{
	public:
		static ResourceManager& get() {return *instance;}
		~ResourceManager();

		void loadResources();

		SDL_RWops* getSample(Sample s) const;
		Texture getTexture(const char* path) const;

		/**
		 * Loads a sequence of textures out of a file.
		 * The textures must all be of the same width (= size) and
		 * placed next to each other in the file.
		 */
		TextureArray getTextures(const char* path) const;

	private:

		/**
		 * converts an SDL_Surface to a OpenGL texture, returns
		 * the texture handle. Only the part specified by the
		 * SDL_Rect will be converted to a texture.
		 * The SDL_Surface will NOT be freed!
		 */
		Texture convertToOpenGlTexture(SDL_Surface* s, SDL_Rect& rect);
		Texture convertToOpenGlTexture(SDL_Surface* s);
		TextureArray loadTextures(const char* file);
		SDL_Surface* loadSurface(const char* path);
		SDL_RWops* loadSample(const char* path);

		typedef std::string String; // using const char* causes trouble
		typedef std::map<String, TextureArray> TextureArrayMap;
		typedef std::map<String, Texture> TextureMap;
		typedef std::map<Sample, SDL_RWops*> SampleMap;
		TextureArrayMap textureArrays;
		TextureMap textures;
		SampleMap samples;

		friend class Engine;
		static ResourceManager* instance;
		ResourceManager();
};



#endif // RESOURCEHANDLER_H
