#include "Scoredisplay.h"
#include "Renderer.h"
#include "ResourceManager.h"
#include "Clock.h"

#include <GL/gl.h>

const char DIGIT_FILE[] = "data/digits.bmp";
const int TEXTURE_SIZE = 32;

Scoredisplay::Scoredisplay()
	: score(0)
{
	digitTextures =  ResourceManager::get().getTextures(DIGIT_FILE);
	addPoints(0); // to set the digits to some value
	Renderer::get().registerAnimator(this);
}

Scoredisplay::~Scoredisplay()
{
	Renderer::get().unregisterAnimator(this);
}

void Scoredisplay::update()
{

}

void Scoredisplay::draw(float dt)
{
	glPushMatrix();
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	for (int i = 0; i < DIGITS; ++i)
	{
		glBindTexture(GL_TEXTURE_2D, digits[i]);
		glBegin(GL_QUADS);
		glTexCoord2f(0, 1);
		glVertex2f(0, 0);
		glTexCoord2f(1, 1);
		glVertex2f(TEXTURE_SIZE, 0);
		glTexCoord2f(1, 0);
		glVertex2f(TEXTURE_SIZE, TEXTURE_SIZE);
		glTexCoord2f(0, 0);
		glVertex2f(0, TEXTURE_SIZE);
		glEnd();
		glTranslatef(TEXTURE_SIZE+1, 0, 0);
	}
	glPopMatrix();


}

void Scoredisplay::addPoints(unsigned int points)
{
	score += points;
	int tmp = score;
	for (int i = DIGITS - 1; i >= 0; --i)
	{
		digits[i] = digitTextures[tmp % 10];
		tmp /= 10;
	}
}


