#include "Shot.h"
#include "Clock.h"
#include "Game.h"
#include "Engine.h"
#include "Zombie.h"
#include "BloodEffect.h"
#include "Sample.h"
#include <cmath>
//#include <iostream>
#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH

// size of a shot (a shot is assumed to be square)
const int SIZE = 3;
const float SPEED = 600.f / Clock::LOGICFRAMES_PER_SECOND;
const int DAMAGE = 10;


Shot::Shot(const Vector2D& pos, float direction_)
	: SpriteObject(SHOT, pos, direction_, Vector2D(SIZE, SIZE)),
	  velocity(SPEED * cos(direction_ * PI/180), SPEED * sin(direction_ * PI/180)),
	  hitSound(PISTOL_HIT),
	  isDead(false)
{
//	std::cout << "Shot " << direction << ' ' << direction_ << '\n';
}

Shot::~Shot()
{
}

bool Shot::isAlive()
{
	return !isDead;
}


void Shot::update()
{
	SpriteObject::update();
	pos += velocity;

	if (pos.x < 0 || pos.y < 0 || pos.x > Engine::WORLDWIDTH || pos.y > Engine::WORLDHEIGHT)
		isDead = true;

	/* collision detection: did we hit anyone?
	 * for every other gameobect, this happens in 2 steps:
	 * a) check if we really hit the object in this frame (ie. our position is within
	 *    the other objects position.
	 * b) check if our line of movement intersects with the line of movement of
	 *    the other object during the time to the next frame. For this we ask for
	 *    the objects position in the next frame and assume linear movement from the
	 *    actual position to there.
	 *    We then calculate the intersection based on the explanation from here:
	 *    http://www.matheraum.de/read?t=64483&v=t
	 *    From that explanation i derived a formula (well, wxMaxima did ;) ) that calculates
	 *    on which t and u values we have the intersection. if both t and u are in [0, 1],
	 *    this counts as a collision. this is not completely exact and the long formulas are
	 *    quite horrible (and actually I don't trust this method of col. detection), but it'll
	 *    have to do for now ;)
	 * TODO: this actually doesn't work as expected. If the shot flies to fast
	 *       it just flies through the zombies w/o the collision being detected
	 */
	Vector2D nextPos(pos + velocity);
	foreach(Zombie* zombie, Game::get().getZombieList())
	{
		const Vector2D& s = zombie->getSize();
		const Vector2D& p = zombie->getPosition();

		if (p.x <= pos.x && p.x + s.x >= pos.x)
		{
			if (p.y <= pos.y && p.y + s.y >= pos.y)
			{
				hit(*zombie);
				isDead = true;
			}
		}

		Vector2D np = zombie->calculateNextPosition();
		float u = (pos.x*(np.y-p.y)+p.x*(pos.y-np.y)+np.x*(p.y-pos.y))/(pos.x*(np.y-p.y)+nextPos.x*(p.y-np.y)+(nextPos.y-pos.y)*np.x+(pos.y-nextPos.y)*p.x);
		float t =-(pos.x*(p.y-nextPos.y)+nextPos.x*(pos.y-p.y)+(nextPos.y-pos.y)*p.x)/(pos.x*(np.y-p.y)+nextPos.x*(p.y-np.y)+(nextPos.y-pos.y)*np.x+(pos.y-nextPos.y)*p.x);
		if (u > 0 && u < 1 && t > 0 && t < 1)
		{
			hit(*zombie);
			isDead = true;
		}
	}
}


Vector2D Shot::calculateNextPosition()
{
	return pos + velocity;
}


void Shot::hit(Pawn& pawn)
{
	pawn.hit(DAMAGE);
	Effect* blood(new BloodEffect(pos, velocity));
	Engine::get().registerEffect(blood);
	hitSound.play();
	return;
}
