#include "SoundManager.h"
#include "EngineError.h"
#include "ResourceManager.h"
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>

const int SAMPLING_RATE = 44100;
const int BUFFER_SIZE = 1024;


namespace
{
Mix_Chunk* loadSample(Sample sample)
{
	SDL_RWops* rwops = ResourceManager::get().getSample(sample);
	Mix_Chunk* tmp = Mix_LoadWAV_RW(rwops, false);
	if (!tmp)
	{
		throw EngineError(std::string("unable to convert RWops"),
			Mix_GetError());
	}

	return tmp;
}
}



SoundManager* SoundManager::instance = 0;


SoundManager::SoundManager()
{
	if (!(SDL_WasInit(SDL_INIT_AUDIO) & SDL_INIT_AUDIO))
	{
		if (SDL_InitSubSystem(SDL_INIT_AUDIO) < 0)
			throw EngineError("unable to initialize the audio subsystem");
	}

	if(Mix_OpenAudio(SAMPLING_RATE, MIX_DEFAULT_FORMAT, 2, BUFFER_SIZE) == -1)
	{
		throw EngineError("unable to initialize the audio mixer");
	}

	Mix_AllocateChannels(1024);
	loadSamples();
}

SoundManager::~SoundManager()
{
	Mix_HaltChannel(-1);
	for (SampleMapIter i = samples.begin(); i != samples.end(); ++i)
	{
		Mix_FreeChunk(i->second);
	}
	SDL_QuitSubSystem(SDL_INIT_AUDIO);
}


void SoundManager::loadSamples()
{
	samples[PISTOL_SHOT] = loadSample(PISTOL_SHOT);
	Mix_VolumeChunk(samples[PISTOL_SHOT], MIX_MAX_VOLUME/4);
	samples[PISTOL_HIT] = loadSample(PISTOL_HIT);

//	samples[ZOMBIE_DEAD]= loadSample("data/zombie_dead.wav");
//	samples[ZOMBIE_DEAD] = loadSample("data/silence.wav");
	samples[PLAYER_SWEAR] = loadSample(PLAYER_SWEAR);
	samples[PLAYER_DEAD]= loadSample(PLAYER_DEAD);
	samples[LEVEL_UP] = loadSample(LEVEL_UP);
}



void SoundManager::play(Sample s) const
{
	SampleMap::const_iterator tmp = samples.find(s);
	if (tmp != samples.end())
	{
		if (Mix_PlayChannel(-1, (tmp->second), 0) < 0)
			throw EngineError("Unable to play sample", Mix_GetError());
	}
	else
		throw EngineError("Unknown sample requested");
}


