#ifndef SPRITE_H
#define SPRITE_H

/**
 * Sprite.h: a basic sprite.
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "typedefs.h"
#include <vector>
#include <boost/shared_ptr.hpp>

class Vector2D;


/**
 * A sprite is a prerendered animation.
 * A Sprite is a sequence of frames. Every frame has a duration,
 * after that expires, the next frame will be displayed.
 * More Animators can share the same Sprite, but every PA has
 * to keep track of the animation-state he is in.
 *
 * NOTE: frames are constructed by the Engine.
 */
class Sprite
{

	public:
		struct State
		{
			public:
				State() : lastFrameChange(0), currentFrame(0) {}
			private:
				friend class Sprite;

				Time lastFrameChange;
				size_t currentFrame;
		};

		void draw(const Vector2D& pos, const Vector2D& size, State* state, float angle) const;
		~Sprite();
	private:

		friend class SpriteManager;
		struct Frame
		{
			Texture texture;
			Time displayTime;
		};

		Sprite();
		std::vector< boost::shared_ptr<Sprite::Frame> > frames;
};

typedef boost::shared_ptr<Sprite> SpritePtr;

#endif
