#include "SpriteManager.h"
#include "Sprite.h"
#include "EngineError.h"
#include "Logger.h"
#include "ResourceManager.h"


#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH

// duration of a single frame (one day we have to make this
// more configurable)
const int FRAME_DURATION = 100;


SpriteManager::SpriteManager()
{
	SpritePtr playerMoving = loadSprite("data/player.bmp");
	SpritePtr playerIdle(new Sprite());
	boost::shared_ptr<Sprite::Frame> f(new Sprite::Frame());
	f->texture = playerMoving->frames[0]->texture;
	f->displayTime = FRAME_DURATION;
	playerIdle->frames.push_back(f);

	sprites[PLAYER_MOVING] = playerMoving;
	sprites[PLAYER_IDLE] = playerIdle;
	sprites[SHOT] = loadSprite("data/shot.bmp");

	SpritePtr zombieMoving = loadSprite("data/zombie.bmp");
	SpritePtr zombieIdle(new Sprite());
	boost::shared_ptr<Sprite::Frame> f2(new Sprite::Frame());
	f2->texture = zombieMoving->frames[0]->texture;
	f2->displayTime = FRAME_DURATION;
	zombieIdle->frames.push_back(f2);

	sprites[ZOMBIE_MOVING] = zombieMoving;
	sprites[ZOMBIE_IDLE] = zombieIdle;
}

SpriteManager::~SpriteManager()
{
	sprites.clear();
}


const Sprite& SpriteManager::getSprite(SpriteType type) const
{
	SpriteMap::const_iterator tmp = sprites.find(type);
	if (tmp != sprites.end())
		return *(tmp->second);
	else
		throw EngineError("Unknown SpriteType requested");
}


SpritePtr SpriteManager::loadSprite(const char* file) const
{
	SpritePtr sprite(new Sprite());

	TextureArray ta = ResourceManager::get().getTextures(file);

	foreach (Texture t, ta)
	{
		boost::shared_ptr<Sprite::Frame> f(new Sprite::Frame());
		f->texture = t;
		f->displayTime = FRAME_DURATION;
		sprite->frames.push_back(f);
	}

	return sprite;
}
