#ifndef SPRITEOBJECT_H
#define SPRITEOBJECT_H


/**
 * SpriteObject.h: Baseclass for all the GameObjects that use sprites.
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "GameObject.h"
#include "SpriteType.h"

class SpriteAnimator;
class Vector2D;



/**
 * Baseclass for all the gameobjects that use sprites to draw
 * themselves (i.e. all the pawns).
 *
 * This class contains it's own pointer to the parent's animator,
 * thus saving the trouble of casting it each time we need a
 * SpriteAnimator specific function.
 * (Besides, all SpriteAnimator-specific stuff should be dealth with
 * within this class).
 */
class SpriteObject : public GameObject
{
	public:
		SpriteObject(SpriteType sprite, const Vector2D& pos,
		             float orientation, const Vector2D& size);

		virtual void update();
		virtual Vector2D calculateNextPosition() = 0;

	protected:
		void changeSprite(SpriteType type);

		float orientation; // orientation in deg

	private:
		SpriteAnimator* spriteAnimator;
};



#endif // SPRITEOBJECT_H
