
#include "Vector2D.h"
#include <cmath>
//#include <iostream>

Vector2D::Vector2D(float x_, float y_)
	: x(x_), y(y_)
{}


void Vector2D::normalize()
{
	if (x*x + y*y == 1) return;
	if (!x && !y) return;
	float len = sqrt(x*x + y*y);
	if (x) x = x/len;
	if (y) y = y/len;
}



float Vector2D::toDegrees() const
{
	float rad = atan2(y, x);
	return rad * 180.f / PI;
}


const Vector2D operator+ (const Vector2D& lhs, const Vector2D& rhs)
{
	return Vector2D(lhs.x + rhs.x, lhs.y + rhs.y);
}

const Vector2D operator- (const Vector2D& lhs, const Vector2D& rhs)
{
	return Vector2D(lhs.x - rhs.x, lhs.y - rhs.y);
}

const Vector2D operator* (const Vector2D& lhs ,float rhs)
{
	return Vector2D(lhs.x * rhs, lhs.y * rhs);
}

const Vector2D operator* (float lhs, const Vector2D& rhs)
{
	return rhs * lhs;
}

const Vector2D operator/ (const Vector2D& lhs, int rhs)
{
	return Vector2D(lhs.x / rhs, lhs.y / rhs);
}

void Vector2D::operator+=(const Vector2D& other)
{
	x += other.x;
	y += other.y;
}


void Vector2D::operator-=(const Vector2D& other)
{
	x -= other.x;
	y -= other.y;
}


void Vector2D::operator *=(float f)
{
	x *= f;
	y *= f;
}


void Vector2D::operator/=(float f)
{
	x /= f;
	y /= f;
}

/*

std::ostream& operator<< (std::ostream& stream, const Vector2D& v)
{
	stream << '(' << v.x << '/' << v.y << ')';
	return stream;
}
*/
